@@ -2,7 +2,10 @@ from __future__ import annotations
 
 import asyncio
 import logging
+from abc import ABC
 from asyncio import Lock, TimerHandle, Transport
+from collections.abc import Callable
+from dataclasses import dataclass
 
 import async_timeout
 
@@ -11,14 +14,31 @@ from .api import RoborockClient
 from .exceptions import RoborockConnectionException, RoborockException
 from .protocol import MessageParser
 from .roborock_message import RoborockMessage, RoborockMessageProtocol
-from .roborock_typing import RoborockCommand
-from .util import RoborockLoggerAdapter
 
 _LOGGER = logging.getLogger(__name__)
 
 
-class RoborockLocalClient(RoborockClient, asyncio.Protocol):
-    def __init__(self, device_data: DeviceData, queue_timeout: int = 4):
+@dataclass
+class _LocalProtocol(asyncio.Protocol):
+    """Callbacks for the Roborock local client transport."""
+
+    messages_cb: Callable[[bytes], None]
+    connection_lost_cb: Callable[[Exception | None], None]
+
+    def data_received(self, bytes) -> None:
+        """Called when data is received from the transport."""
+        self.messages_cb(bytes)
+
+    def connection_lost(self, exc: Exception | None) -> None:
+        """Called when the transport connection is lost."""
+        self.connection_lost_cb(exc)
+
+
+class RoborockLocalClient(RoborockClient, ABC):
+    """Roborock local client base class."""
+
+    def __init__(self, device_data: DeviceData):
+        """Initialize the Roborock local client."""
         if device_data.host is None:
             raise RoborockException("Host is required")
         self.host = device_data.host
@@ -28,17 +48,19 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
         self.transport: Transport | None = None
         self._mutex = Lock()
         self.keep_alive_task: TimerHandle | None = None
-        self._logger = RoborockLoggerAdapter(device_data.device.name, _LOGGER)
-        RoborockClient.__init__(self, "abc", device_data, queue_timeout)
+        RoborockClient.__init__(self, device_data)
+        self._local_protocol = _LocalProtocol(self._data_received, self._connection_lost)
 
-    def data_received(self, message):
+    def _data_received(self, message):
+        """Called when data is received from the transport."""
         if self.remaining:
             message = self.remaining + message
             self.remaining = b""
         parser_msg, self.remaining = MessageParser.parse(message, local_key=self.device_info.device.local_key)
         self.on_message_received(parser_msg)
 
-    def connection_lost(self, exc: Exception | None):
+    def _connection_lost(self, exc: Exception | None):
+        """Called when the transport connection is lost."""
         self.sync_disconnect()
         self.on_connection_lost(exc)
 
@@ -61,7 +83,7 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
                     async with async_timeout.timeout(self.queue_timeout):
                         self._logger.debug(f"Connecting to {self.host}")
                         self.transport, _ = await self.event_loop.create_connection(  # type: ignore
-                            lambda: self, self.host, 58867
+                            lambda: self._local_protocol, self.host, 58867
                         )
                         self._logger.info(f"Connected to {self.host}")
                         should_ping = True
@@ -96,7 +118,7 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
         except Exception as e:
             self._logger.error(e)
 
-    async def ping(self):
+    async def ping(self) -> None:
         request_id = 2
         protocol = RoborockMessageProtocol.PING_REQUEST
         return await self.send_message(
@@ -107,13 +129,6 @@ class RoborockLocalClient(RoborockClient, asyncio.Protocol):
             )
         )
 
-    async def _send_command(
-        self,
-        method: RoborockCommand | str,
-        params: list | dict | int | None = None,
-    ):
-        raise NotImplementedError
-
     def _send_msg_raw(self, data: bytes):
         try:
             if not self.transport:
